/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995, 1996, 1997, 1998 John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@johncon.com (John Conover)

------------------------------------------------------------------------------

uppercase.c, uppercase and soundex transliteration

void make_uppercase (void);

    construct the arrays, uppercase and soundex, in such a manner that
    the implicit index of any element in the array contains the
    toupper() or the soundex value of the index value (ie., it is a
    look up table for uppercase and soundex characters.)

    note: care must be exercised when using this array in systems
    where the native type of char is signed, for example:

        signed char ch;

        unsigned char cu;

        cu = uppercase[ch];

    will not give the desired results, since ch indexed a negative
    section of the array, (which does not exist.). Particularly
    meticulous usage of lint is advisable.

    The objective of this technique is to provide an alternative to
    using toupper() and computing the soundex value of a letter via
    branch constructs, on every character in a record-implicit
    indexing is very fast, and once the uppercase and soundex arrays
    have been set up, uppercase/soundex transliteration of records can
    be made very quickly. As a related issue, it should be,
    relatively, portable to other locale.h environments. It is
    probably important to note that the characters in any keyword
    search criteria of records and the records them selves should both
    be transliterated using these arrays so that the character sets
    for both are identical.

    Other transliterations can be placed in the array, for example,
    tabs and newlines can be converted to EOS's, (which is the current
    implementation,) as could punctuation such as commas and periods,
    (which are, also, converted to EOS's in this implementation.) A
    possible scenario would be to use ispunct(), iswhite(), isprint(),
    etc. in the for loop to alter the transliteration to whatever is
    desired-be advised that portability issues may ensue if the scheme
    is not compatible with local.h in some languages. A possible
    alternative would be to implement a command line switch for
    various languages, to avoid localization and portability
    conflicts.

The algorithm is as follows:

    for each element in the uppercase array, store the toupper() of
    the index of the element, and the soundex translation for the
    index of the element

Usage is a single call to allocate and scan the array, for example:

    unsigned char my_array_of_uppercase[],
                  *my_ptr;

    make_uppercase ();

    while (something)
    {
        my_array_of_uppercase[something] = uppercase[(int) *my_ptr];
    }

For a detailed description of using implicit addressing for character
transliteration, see "Information Retrieval: Data Structures &
Algorithms," William B. Frakes, Ricardo Baeza-Yates, Editors, Prentice
Hall, Englewood Cliffs, New Jersey, 1992, ISBN 0-13-463837-9, pp 102.

There are no arguments

There are no errors, returns nothing

MAX_ALPHABET_SIZE is defined in uppercase.h

To test this module, compile the module source with -DTEST_UPPERCASE

$Revision: 1.0 $
$Date: 1998/03/09 00:29:06 $
$Id: uppercase.c,v 1.0 1998/03/09 00:29:06 john Exp $
$Log: uppercase.c,v $
Revision 1.0  1998/03/09 00:29:06  john
Initial revision


*/

#include "pdict.h"

#ifndef LINT /* include rcsid only if not running lint */

static char rcsid[] = "$Id: uppercase.c,v 1.0 1998/03/09 00:29:06 john Exp $"; /* module version */
static char rcsid_h[] = UPPERCASE_H_ID; /* module include version */

#endif

unsigned char uppercase[MAX_ALPHABET_SIZE]; /* uppercase array */

unsigned char soundex[MAX_ALPHABET_SIZE]; /* soundex array */

#ifdef __STDC__

void make_uppercase (void)

#else

void make_uppercase ()

#endif

{
    int i; /* character counter */

    for (i = 0; i < MAX_ALPHABET_SIZE; i++) /* for each character in the uppercase array */
    {

        if (isalpha (i)) /* character alphabetical? */
        {
            uppercase[i] = toupper (i); /* yes, save the uppercase value of the character */

            switch ((int) uppercase[i]) /* translate the uppercase value to its soundex characer */
            {

                case (int) 'B': /* replace sequential labials with a single '1' */
                case (int) 'F':
                case (int) 'P':
                case (int) 'V':

                    soundex[i] = (unsigned char) '1'; /* make the soundex character a labial */
                    break;

                case (int) 'C': /* replace sequential gutterals and sibilants with a single '2' */
                case (int) 'G':
                case (int) 'J':
                case (int) 'K':
                case (int) 'Q':
                case (int) 'S':
                case (int) 'X':
                case (int) 'Z':

                    soundex[i] = (unsigned char) '2'; /* make the soundex character a gutteral or sibilant */
                    break;

                case (int) 'D': /* replace sequential dentals with a single '3' */
                case (int) 'T':

                    soundex[i] = (unsigned char) '3'; /* make the soundex character a dental */
                    break;

                case (int) 'L': /* replace sequential longliquids with a single '4' */

                    soundex[i] = (unsigned char) '4'; /* make the soundex character a longliquid */
                    break;

                case (int) 'M': /* replace sequential nasals with a single '5' */
                case (int) 'N':

                    soundex[i] = (unsigned char) '5'; /* make the soundex character a nasal */
                    break;

                case (int) 'R': /* replace sequential shortliquids with a single '6' */

                    soundex[i] = (unsigned char) '6'; /* make the soundex character a shortliquid */
                    break;

                case (int) 'A': /* skip the rest of the alphabeticals */
                case (int) 'E':
                case (int) 'H':
                case (int) 'I':
                case (int) 'O':
                case (int) 'U':
                case (int) 'W':
                case (int) 'Y':

                    soundex[i] = (unsigned char) '\0'; /* make the soundex character an EOS */
                    break;

                default: /* any other character? */

                    soundex[i] = (unsigned char) '\0'; /* make the soundex character an EOS */
                    break;

            }

        }

        else
        {
            uppercase[i] = (unsigned char) '\0'; /* no, treat the charcter as an EOS character in both the uppercase and soundex arrays */
        }

    }

}

#ifdef TEST_UPPERCASE

/*

simple exerciser for testing make_uppercase (); dump the uppercase and
soundex arrays to stdio

declared global, could be static
    uppercase           uppercase.c(xxx)
    make_uppercase      uppercase.c(yyy)

from lint

*/

#ifdef __STDC__

int main (void)

#else

int main ()

#endif

{
    int i; /* character counter */

    make_uppercase (); /* setup the uppercase array */

    for (i = 0; i < MAX_ALPHABET_SIZE; i++) /* for each character in the uppercase and soundex array */
    {
        (void) printf ("uppercase[%d], uppercase = %d, soundex = %d\n", i, (int) uppercase[i], (int) soundex[i]); /* print the character's decimal value to stdio */
    }

    exit (0); /* return success */

#ifdef LINT /* include only if running lint */

    return (0); /* for LINT formality */

#endif

}

#endif
